unit modul_pio;

{$mode objfpc}

interface

uses
  Classes, SysUtils, DateUtils;



const

{
; PIO OUT
PIO_OUT_CONOUT		EQU	001H
PIO_OUT_RW_VECTOR_LO	EQU	002H
PIO_OUT_READ_VECTOR_HI	EQU	003H
PIO_OUT_WRITE_VECTOR_HI	EQU	004H
PIO_OUT_LIST		EQU	005H
PIO_OUT_LOAD_BDOS	EQU	006H
PIO_OUT_LOAD_CCP	EQU	007H
PIO_OUT_DT_INIT         EQU	008H

PIO_OUT_HALT		EQU	0FFH
}

  // PIO OUT
  PIO_OUT_CONOUT           = $001;
  PIO_OUT_RW_VECTOR_LO     = $002;
  PIO_OUT_READ_VECTOR_HI   = $003;
  PIO_OUT_WRITE_VECTOR_HI  = $004;
  PIO_OUT_LIST             = $005;
  PIO_OUT_LOAD_BDOS        = $006;
  PIO_OUT_LOAD_CCP         = $007;

  PIO_OUT_DT_INIT          = $008;

  PIO_OUT_HALT             = $0FF;


{
; PIO IN
PIO_IN_CONST		EQU	001H
PIO_IN_CONIN		EQU	002H
PIO_IN_DT_READ          EQU	003H
}

  // PIO IN
  PIO_IN_CONST             = $001;
  PIO_IN_CONIN             = $002;
  PIO_IN_DT_READ           = $003;



// DRV_VEC

OFFSET_SECTOR = 0;
OFFSET_TRACK  = 1;
OFFSET_DISKNO = 3;
OFFSET_DMAAD  = 4;




type


  Tmod_pio = class(TObject)

    constructor Create; overload;
    destructor Destroy; override;

  private
    RW_VECTOR_LO : Byte;
    DT_String : String;
    function CONST_auslesen : Byte;
    function CONIN_auslesen : Byte;

  public
    CONIN_Buffer : String;

    function Input_PIO(Port : Byte) : Byte; { ein Byte von der PIO lesen }
    procedure Output_PIO(Port, Data : Byte); { ein Byte in die PIO schreiben }

  end; // Tmod_pio = class(TObject)




var
  mod_pio: Tmod_pio;



implementation


uses
  MainForm_EMUZ80, MainForm_buttons, modul_ram, modul_cpu;



constructor Tmod_pio.Create;
begin
  inherited Create;

  CONIN_Buffer:='';
  DT_String:='329B133357';

end; { constructor Tmod_pio.Create }







function Tmod_pio.CONST_auslesen : Byte;
var
  Ergebnis : Byte;
begin
  Ergebnis:=0;

  if Length(CONIN_Buffer)>0 then begin

    Ergebnis:=$FF;

  end; { if Length(CONIN_Buffer)>0 then }

  Result := Ergebnis;

end; { function Tmod_pio.CONST_auslesen }





function Tmod_pio.CONIN_auslesen : Byte;
var
  Ergebnis : Byte;
begin
  Ergebnis:=0;

  if Length(CONIN_Buffer)>0 then begin

    Ergebnis:=Byte(CONIN_Buffer[1]);
    Delete(CONIN_Buffer,1,1);

    Form_EMUZ80.Caption := CONIN_Buffer;

  end; { if Length(CONIN_Buffer)>0 then }

  Result := Ergebnis;

end; { function Tmod_pio.CONIN_auslesen }






function Tmod_pio.Input_PIO(Port : Byte) : Byte; { ein Byte von der PIO lesen }
var
  Ergebnis, Ergebnis_LO, Ergebnis_HI, Stelle : Byte;
  Zeichen : Char;

begin
  Ergebnis := 0;

  {
  PIO_IN_CONST		EQU	001H
  PIO_IN_CONIN		EQU	002H
  PIO_IN_DT_READ        EQU	003H

  }

  case Port of

    PIO_IN_CONST             : begin
                                 Ergebnis:=CONST_auslesen;
                               end; { PIO_IN_CONST }

    PIO_IN_CONIN             : begin
                                 Ergebnis:=CONIN_auslesen;
                               end; { PIO_IN_CONIN }

    PIO_IN_DT_READ           : begin

                                 if (mod_cpu.Reg.HRS.C>0) and (mod_cpu.Reg.HRS.C<6) then begin

                                   Stelle:=mod_cpu.Reg.HRS.C;
                                   Stelle:=Stelle-1;
                                   Stelle:=Stelle*2;
                                   Inc(Stelle);
                                   Zeichen:=DT_String[Stelle];
                                   if (Zeichen>='0') and (Zeichen<='9') then begin
                                     Ergebnis_HI:=Byte(Zeichen)-$30;
                                   end
                                   else begin
                                     if (Zeichen>='A') and (Zeichen<='F') then begin
                                       Ergebnis_HI:=Byte(Zeichen)-$41+10;
                                     end
                                     else begin

                                       TForm_BUTTONS.Memo_Protokoll.Lines.Add(
                                         'Fehler Input-Port, PIO_IN_DT_READ, Hexawert falsch ('+
                                         Zeichen+')');

                                     end;
                                   end;

                                   Inc(Stelle);
                                   Zeichen:=DT_String[Stelle];
                                   if (Zeichen>='0') and (Zeichen<='9') then begin
                                     Ergebnis_LO:=Byte(Zeichen)-$30;
                                   end
                                   else begin
                                     if (Zeichen>='A') and (Zeichen<='F') then begin
                                       Ergebnis_LO:=Byte(Zeichen)-$41+10;
                                     end
                                     else begin

                                       TForm_BUTTONS.Memo_Protokoll.Lines.Add(
                                         'Fehler Input-Port, PIO_IN_DT_READ, Hexawert falsch ('+
                                         Zeichen+')');

                                     end;
                                   end;
                                   Ergebnis:=Ergebnis_LO + (Ergebnis_HI * 16);


                                 end
                                 else begin
                                   Ergebnis:=0;
                                   TForm_BUTTONS.Memo_Protokoll.Lines.Add(
                                     'Fehler Input-Port, PIO_IN_DT_READ, Wert in Register C muss zwischen 1 und 10 liegen ('+
                                     IntToStr(mod_cpu.Reg.HRS.C)+')');
                                 end;

                               end; { PIO_IN_DT_READ }


  else

    TForm_BUTTONS.Memo_Protokoll.Lines.Add('Input-Port:'+IntToStr(Port)+' wird nicht ausgewertet');

  end; { case Adress of }

  Result:=Ergebnis;

end; { function Tmod_pio.Read_Byte }





procedure Tmod_pio.Output_PIO(Port, Data : Byte); { ein Byte in die PIO schreiben }
var
  RW_VECTOR_HI, Byte_Low, Byte_High : Byte;
  RW_VECTOR, VAR_DMAAD : Word;
  VEC_Zaehler : Integer;



        procedure schreibe_Datum_und_Zeit_in_DT_String;
        var
          Start_Datum : TDateTime;
          Zeile, S_Stunde, S_Minute, S_Sekunde : String;
          Diff : Word;
        begin
          Start_Datum:=StrToDate('01.01.1978');
          Diff:=DaysBetween(Date, Start_Datum)+1;
          Zeile:=TimeToStr(Time);
          S_Stunde:=Copy(Zeile,1,2);
          S_Minute:=Copy(Zeile,4,2);
          S_Sekunde:=Copy(Zeile,7,2);

          // BD35112542 = Datum(4) + Zeit(6) = BD35 -> 31.08.2015; 112542 -> 11:25:42 Uhr

          DT_String:=IntToHex(Diff,4);
          DT_String:=DT_String[3]+DT_String[4]+DT_String[1]+DT_String[2]+S_Stunde+S_Minute+S_Sekunde;

        end; { schreibe_Datum_und_Zeit_in_DT_String  }



begin

  {
  PIO_OUT_CONOUT           : Byte = $001;
  PIO_OUT_RW_VECTOR_LO     : Byte = $002;
  PIO_OUT_READ_VECTOR_HI   : Byte = $003;
  PIO_OUT_WRITE_VECTOR_HI  : Byte = $004;
  PIO_OUT_LIST             : Byte = $005;
  PIO_OUT_LOAD_BDOS        : Byte = $006;
  PIO_OUT_LOAD_CCP         : Byte = $007;
  PIO_OUT_DT_INIT          : Byte = $008;
  PIO_OUT_HALT             : Byte = $0FF;
  }

  case Port of

    PIO_OUT_CONOUT           : begin
                                 Form_EMUZ80.Zeichen_an_CONOUT(Data);
                               end; { PIO_OUT_CONOUT }

    PIO_OUT_RW_VECTOR_LO     : begin
                                 RW_VECTOR_LO:=Data;
                               end; { PIO_OUT_RW_VECTOR_LO }


    PIO_OUT_READ_VECTOR_HI   : begin
                                 RW_VECTOR_HI:=Data;
                                 RW_VECTOR := (RW_VECTOR_HI shl 8) + RW_VECTOR_LO;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Form_EMUZ80.VAR_SECTOR := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Form_EMUZ80.VAR_TRACK := (Byte_High shl 8) + Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Form_EMUZ80.VAR_DISKNO := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 VAR_DMAAD := (Byte_High shl 8) + Byte_Low;

                                 for VEC_Zaehler:=0 to 3 do begin

                                   Form_EMUZ80.Read_IMAGE_Sektor(VEC_Zaehler, VAR_DMAAD);

                                 end;

                               end; { PIO_OUT_READ_VECTOR_HI }



    PIO_OUT_WRITE_VECTOR_HI  : begin
                                 RW_VECTOR_HI:=Data;
                                 RW_VECTOR := (RW_VECTOR_HI shl 8) + RW_VECTOR_LO;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Form_EMUZ80.VAR_SECTOR := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Form_EMUZ80.VAR_TRACK := (Byte_High shl 8) + Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Form_EMUZ80.VAR_DISKNO := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 VAR_DMAAD := (Byte_High shl 8) + Byte_Low;

                                 for VEC_Zaehler:=0 to 3 do begin

                                   Form_EMUZ80.Write_IMAGE_Sektor(VEC_Zaehler, VAR_DMAAD);

                                 end;

                               end; { PIO_OUT_WRITE_VECTOR_HI }


    PIO_OUT_LIST             : begin
                                 // erst mal nur Dummy

                               end; { PIO_OUT_LIST }



    PIO_OUT_LOAD_BDOS        : begin
                                 Form_EMUZ80.Lade_BDOS_in_RAM(Data);
                               end; { PIO_OUT_LOAD_BDOS }



    PIO_OUT_LOAD_CCP         : begin
                                 Form_EMUZ80.Lade_CCP_in_RAM(Data);
                               end; { PIO_OUT_LOAD_CCP }



    PIO_OUT_DT_INIT          : begin
                                 schreibe_Datum_und_Zeit_in_DT_String;
                                 // TForm_BUTTONS.Memo_Protokoll.Lines.Add('DT_String='+DT_String);
                               end; { PIO_OUT_DT_INIT }



    PIO_OUT_HALT             : begin
                                 TForm_BUTTONS.Memo_Protokoll.Lines.Add('PIO_OUT_HALT');
                               end; { PIO_OUT_HALT }


  else

    TForm_BUTTONS.Memo_Protokoll.Lines.Add('Output-Port:'+IntToStr(Port)+' wird nicht ausgewertet');

  end; { case Adress of }


end; { procedure Tmod_pio.Write_Byte }





destructor Tmod_pio.Destroy;
begin

  inherited Destroy;
end; { destructor Tmod_pio.Destroy }



end.

